// Sunrise Alarm Clock Project
//
// This is the implimentation for the AdcDriver code
//
// required stm library code:
// 	stm3210x_adc.c
//  stm3210x_rcc.c
//
// Also the APB2 peripheral clocks must be enabled for: 
//  GPIOC 
//  ADC1EN 
//  ADC2EN
//
// Allen Kennedy
// 4/15/2011

#include <stm32f10x_lib.h>

/******************************************************************************
*       NAME : deinitLightSensor(void)
*
*DESCRIPTION : Destructor for the light sensor
*
*     INPUTS : None
*
*     OUTPUT : None
*
*    RETURNS : None
*
******************************************************************************/
void deinitLightSensor()
{
	// disable the GPIO for sensor power.  
	// *****************************************************
	// WARNING THIS DISABLES ZERO CROSSING SENSOR ALSO!
	// perhaps this should be a seperate funtion?
	// *****************************************************
	GPIOC->CRL &= 0xFFF0FFFF;
	GPIOC->CRL |= 0x00030000;

	GPIOC->ODR |=  0x00000010;

	// stop conversion ADC1
	ADC_SoftwareStartConvCmd(ADC2, DISABLE);

	// Disable ADC1 
	ADC_Cmd(ADC2, DISABLE);

	// ADC1 configuration 
	ADC_DeInit(ADC2);
}

/******************************************************************************
*       NAME : initLightSensor(void)
*
*DESCRIPTION : Initializer for the light sensor
*
*     INPUTS : None
*
*     OUTPUT : None
*
*    RETURNS : None
*
******************************************************************************/
void initLightSensor()
{
	ADC_InitTypeDef ADC_InitStructure;

	// set up the GPIO for sensor power.
	GPIOC->CRL &= 0xFFF0FFFF;
	GPIOC->CRL |= 0x00030000;

	GPIOC->ODR |=  0x00000010;

	// enable ADC1
	ADC_SoftwareStartConvCmd(ADC2, DISABLE);
	
	// ADC1 configuration 
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC2, &ADC_InitStructure);
	
	// ADC1 regular channel15 (PC5) configuration 
	ADC_RegularChannelConfig(ADC2, ADC_Channel_15, 1, ADC_SampleTime_55Cycles5);
	
	// Enable ADC1 
	ADC_Cmd(ADC2, ENABLE);
	
	// Enable ADC1 reset calibaration register 
	ADC_ResetCalibration(ADC2);
	// Check the end of ADC1 reset calibration register 
	while(ADC_GetResetCalibrationStatus(ADC2));
	
	// Start ADC1 calibaration 
	ADC_StartCalibration(ADC2);
	// Check the end of ADC1 calibration 
	while(ADC_GetCalibrationStatus(ADC2));
	 
	// Start ADC1 Software Conversion 
	ADC_SoftwareStartConvCmd(ADC2, ENABLE);

	// wait for at least one conversion to finish
	while(ADC_GetFlagStatus(ADC2, ADC_FLAG_EOC) == 0);
}


/******************************************************************************
*       NAME : getLightSensorReading(void)
*
*DESCRIPTION : Initializer for the Temperature sensor
*
*     INPUTS : None
*
*     OUTPUT : None
*
*    RETURNS : None
*
******************************************************************************/
unsigned short getLightSensorReading()
{
	return ADC_GetConversionValue(ADC2);
}


/******************************************************************************
*       NAME : initTempSensor(void)
*
*DESCRIPTION : Returns back teh temperature in degrees Farhenhight
*
*     INPUTS : None
*
*     OUTPUT : None
*
*    RETURNS : unsigned short - Value from 0 to 212
*
******************************************************************************/
void initTempSensor()
{
	ADC_InitTypeDef ADC_InitStructure;

	// enable ADC2
	ADC_SoftwareStartConvCmd(ADC1, DISABLE);
	
	// ADC1 configuration 
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);
	
	// ADC1 regular temperature configuration 
	ADC_RegularChannelConfig(ADC1, ADC_Channel_16, 1, ADC_SampleTime_239Cycles5);
	
	// Enable ADC1 
	ADC_Cmd(ADC1, ENABLE);

	// Turn on the Temp Sensor Reference Voltage
	ADC_TempSensorVrefintCmd(ENABLE);
	
	// Enable ADC1 reset calibaration register 
	ADC_ResetCalibration(ADC1);
	// Check the end of ADC1 reset calibration register 
	while(ADC_GetResetCalibrationStatus(ADC1));
	
	// Start ADC1 calibaration 
	ADC_StartCalibration(ADC1);
	// Check the end of ADC1 calibration 
	while(ADC_GetCalibrationStatus(ADC1));
	 
	// Start ADC1 Software Conversion 
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);

	// wait for at least one conversion to finish
	while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == 0);
}

/******************************************************************************
*       NAME : getTemperature(void)
*
*DESCRIPTION : Returns back teh temperature in degrees Farhenhight
*
*     INPUTS : None
*
*     OUTPUT : None
*
*    RETURNS : unsigned short - Value from 0 to 212
*
******************************************************************************/
unsigned short getTemperature()
{
	u16 temp;
	// get the ADC reading
	u32 rawAdc = ADC_GetConversionValue(ADC1);

	// shift it up to help remove some of the error when doing the 
	// math for conversion.	 a factor of 256.
	rawAdc <<= 8;

/*Temperature (in C) = {(V25 - VSENSE) / Avg_Slope} + 25.
Where,
V25 = VSENSE value for 25 C and
Avg_Slope = Average Slope for curve between Temperature vs. VSENSE (given in
mV/ C or V/ C).
Refer to the Electrical characteristics section for the actual values of V25 and
Avg_Slope.

V25 = 1.41  times 256 = 361
Avg_slope = 4.3		  = 1101

1.41 = (x/4096)*Vref where Vref = 3.3 for this package
x = (1.41/3.3)*4096;
1750.11 * 256 = 448028, which in hex is    0x6d61c

*/

	temp = ((361 - rawAdc) / 1101 ) + 25;



	return 71;
}

/******************************************************************************
*       NAME : deinitTempSensor(void)
*
*DESCRIPTION : Destructor for the Temperature sensor
*
*     INPUTS : None
*
*     OUTPUT : None
*
*    RETURNS : None
*
******************************************************************************/
void deinitTempSensor()
{

}

